@@ -41,8 +41,13 @@ oneTimeTearDown() {
 }
 
 setUp() {
-	GIT_PROJECT_PATH=$(mktemp -d -t git-ftp-XXXX)
-	GIT_PROJECT_NAME=$(basename $GIT_PROJECT_PATH)
+	if command -v mktemp > /dev/null 2>&1; then
+		GIT_PROJECT_PATH="$(mktemp -d -t git-ftp-XXXX)"
+	else
+		GIT_PROJECT_PATH="git-ftp-test-repo-$(date | md5sum | cut -d ' ' -f1)"
+		mkdir -p "$GIT_PROJECT_PATH"
+	fi
+	GIT_PROJECT_NAME="$(basename $GIT_PROJECT_PATH)"
 
 	GIT_FTP_URL="$GIT_FTP_ROOT/$GIT_PROJECT_NAME"
 
@@ -69,6 +74,8 @@ setUp() {
 }
 
 tearDown() {
+	tmpfiles=$(ls .git-ftp*-tmp 2> /dev/null)
+	assertEquals '' "$tmpfiles"
 	rm -rf $GIT_PROJECT_PATH
 	command -v lftp >/dev/null 2>&1 && {
 		lftp -u $GIT_FTP_USER,$GIT_FTP_PASSWD $GIT_FTP_ROOT -e "set ftp:list-options -a; rm -rf '$GIT_PROJECT_NAME'; exit" > /dev/null 2>&1
@@ -82,7 +89,7 @@ test_displays_usage() {
 
 test_prints_version() {
 	version=$($GIT_FTP_CMD 2>&1 --version)
-	assertEquals = "git-ftp version 1.0.2"  "$version"
+	assertEquals = "git-ftp version 1.1.0"  "$version"
 }
 
 test_inits() {
@@ -123,71 +130,6 @@ test_inits_and_pushes() {
 	assertEquals 0 $rtrn
 }
 
-test_init_more() {
-	cd $GIT_PROJECT_PATH
-
-	# Generate a number of files exceeding the upload buffer
-	long_prefix='aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'
-	long_file_list=''
-	for i in `seq 50`; do
-		long_file_list="$long_file_list $long_prefix$i"
-	done
-	touch $long_file_list
-	git add .
-	git commit -m 'Some more files.' > /dev/null
-
-	init=$($GIT_FTP init)
-	assertEquals 0 $?
-	assertTrue 'file does not exist' "remote_file_exists '${long_prefix}50'"
-
-	# Counting the number of uploads to estimate correct buffering
-	upload_count=$(echo "$init" | grep -Fx 'Uploading ...' | wc -l)
-	assertTrue "[ $upload_count -gt 1 ]"
-}
-
-test_delete_more() {
-	cd $GIT_PROJECT_PATH
-
-	# Generate a number of files exceeding the upload buffer
-	long_prefix='aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'
-	long_file_list=''
-	for i in `seq 50`; do
-		long_file_list="$long_file_list $long_prefix$i"
-	done
-	touch $long_file_list
-	git add .
-	git commit -m 'Some more files.' > /dev/null
-
-	init=$($GIT_FTP init)
-	assertEquals 0 $?
-
-	# Delete a number of files exceeding the upload buffer
-	git rm $long_file_list > /dev/null
-	git commit -m 'Deleting some more files.' > /dev/null
-
-	push=$($GIT_FTP push)
-	assertEquals 0 $?
-
-	# Counting the number of deletes to estimate correct buffering
-	delete_count=$(echo "$push" | grep -Fx 'Deleting ...' | wc -l)
-	assertTrue "[ $delete_count -gt 1 ]"
-	assertFalse 'file does exist' "remote_file_exists '${long_prefix}50'"
-}
-
-# this test takes a couple of minutes (revealing a performance issue)
-disabled_test_init_heaps() {
-	cd $GIT_PROJECT_PATH
-
-	# Generate a large number of files which fails to upload on some systems
-	touch `seq 3955`
-	git add .
-	git commit -m 'A lot of files.' > /dev/null
-
-	$GIT_FTP init> /dev/null
-	assertEquals 0 $?
-	assertTrue 'file does not exist' "remote_file_exists '3955'"
-}
-
 test_pushes_and_fails() {
 	cd $GIT_PROJECT_PATH
 	push="$($GIT_FTP push 2>&1)"
@@ -421,7 +363,7 @@ test_delete() {
 
 test_ignore_single_file() {
 	cd $GIT_PROJECT_PATH
-	echo "test 1\.txt" > .git-ftp-ignore
+	echo "test 1.txt" > .git-ftp-ignore
 
 	init=$($GIT_FTP init)
 
@@ -442,7 +384,7 @@ test_ignore_single_file_force_unknown_commit() {
 
 test_ignore_dir() {
 	cd $GIT_PROJECT_PATH
-	echo "dir 1/.*" > .git-ftp-ignore
+	echo "dir 1/*" > .git-ftp-ignore
 
 	init=$($GIT_FTP init)
 
@@ -452,7 +394,7 @@ test_ignore_dir() {
 
 test_ignore_pattern() {
 	cd $GIT_PROJECT_PATH
-	echo "test" > .git-ftp-ignore
+	echo "test*" > .git-ftp-ignore
 
 	init=$($GIT_FTP init)
 
@@ -462,26 +404,10 @@ test_ignore_pattern() {
 	done;
 }
 
-disabled_test_ignore_pattern_single() {
+test_ignore_pattern_single() {
 	cd $GIT_PROJECT_PATH
 	echo 'test' > 'test'
-	echo "^test$" > .git-ftp-ignore
-	git add .
-	git commit -m 'adding file that should not be uploaded' > /dev/null
-
-	init=$($GIT_FTP init)
-
-	assertFalse 'test failed: was not ignored' "remote_file_exists 'test'"
-	for i in 1 2 3 4 5
-	do
-		assertTrue 'test failed: was ignored' "remote_file_exists 'test $i.txt'"
-	done;
-}
-
-# TODO: make this test fail an the previous test work
-test_ignore_pattern_single_dirty() {
-	echo 'test' > 'test'
-	echo '^. test$' > .git-ftp-ignore
+	echo 'test' > .git-ftp-ignore
 	git add .
 	git commit -m 'adding file that should not be uploaded' > /dev/null
 
@@ -496,7 +422,7 @@ test_ignore_pattern_single_dirty() {
 
 test_ignore_wildcard_files() {
 	cd $GIT_PROJECT_PATH
-	echo "test.*\.txt" > .git-ftp-ignore
+	echo "test *.txt" > .git-ftp-ignore
 
 	init=$($GIT_FTP init)
 
@@ -529,6 +455,17 @@ test_include_directory() {
 	assertFalse 'unversioned-not-included/file.txt was uploaded' "remote_file_exists 'unversioned-not-included/file.txt'"
 }
 
+test_include_directory_always() {
+	mkdir unversioned
+	touch unversioned/file.txt
+	echo '!unversioned/' > .git-ftp-include
+	mkdir unversioned-not-included
+	touch unversioned-not-included/file.txt
+	init=$($GIT_FTP init)
+	assertTrue 'unversioned/file.txt was not uploaded' "remote_file_exists 'unversioned/file.txt'"
+	assertFalse 'unversioned-not-included/file.txt was uploaded' "remote_file_exists 'unversioned-not-included/file.txt'"
+}
+
 test_include_whitespace_init() {
 	cd $GIT_PROJECT_PATH
 	echo 'unversioned' > unversioned.txt
@@ -553,6 +490,26 @@ test_include_push() {
 	assertTrue 'unversioned.txt was not uploaded' "remote_file_exists 'unversioned.txt'"
 }
 
+test_include_push_delete() {
+	echo 'unversioned' > unversioned.txt
+	echo 'unversioned.txt' >> .gitignore
+	echo 'unversioned.txt:test 1.txt' > .git-ftp-include
+	echo 'unversioned.txt:test 2.txt' >> .git-ftp-include
+	echo 'new content' >> 'test 1.txt'
+	git add .
+	git commit -m 'unversioned file unversioned.txt should be uploaded with test 1.txt' -q
+	init=$($GIT_FTP init)
+	git rm 'test 1.txt' -q
+	git commit -m 'the trigger file of unversioned.txt is deleted which deletes the target file' -q
+	push=$($GIT_FTP push)
+	assertTrue 'unversioned.txt was deleted' "remote_file_exists 'unversioned.txt'"
+	echo 'new content' >> 'test 2.txt'
+	rm 'unversioned.txt'
+	git commit -a -m 'the local deletion of unversioned.txt should delete remote file' -q
+	push=$($GIT_FTP push)
+	assertFalse 'unversioned.txt was not deleted' "remote_file_exists 'unversioned.txt'"
+}
+
 test_include_ignore_init() {
 	cd $GIT_PROJECT_PATH
 	echo 'htaccess' > .htaccess
@@ -632,20 +589,25 @@ test_hidden_file_only() {
 }
 
 
+# issue #23
 test_file_with_nonchar() {
-	cd $GIT_PROJECT_PATH
-	echo "test" > ./#4253-Release Contest.md
-	git add . > /dev/null 2>&1
-	git commit -a -m "init" > /dev/null 2>&1
-
+	file1='#4253-Release Contest.md'
+	file1enc='%234253-Release%20Contest.md'
+	file2='v1.2.0 #8950 - Custom Partner Player.md'
+	file2enc='v1.2.0%20%238950%20-%20Custom%20Partner%20Player.md'
+	echo 'content1' > "$file1"
+	echo 'content2' > "$file2"
+	git add .
+	git commit -a -m 'added special filenames' -q
 	init=$($GIT_FTP init)
-	assertTrue 'test failed: #4253-Release Contest.md not uploaded' "remote_file_exists '#4253-Release Contest.md'"
-
-	git rm './#4253-Release Contest.md' > /dev/null 2>&1
-	git commit -a -m "delete" > /dev/null 2>&1
-
+	assertTrue " file $file1 not uploaded" "remote_file_equals '$file1' '$file1enc'"
+	assertTrue " file $file2 not uploaded" "remote_file_equals '$file2' '$file2enc'"
+	git rm "$file1" -q
+	git rm "$file2" -q
+	git commit -m 'delete' -q
 	push=$($GIT_FTP push)
-	assertFalse 'test failed: #4253-Release Contest.md still exists in '$CURL_URL "remote_file_exists '\#4253-Release Contest.md'"
+	assertFalse "file $file1 still exists in $CURL_URL" "remote_file_exists '$file1enc'"
+	assertFalse "file $file2 still exists in $CURL_URL" "remote_file_exists '$file2enc'"
 }
 
 test_syncroot() {
@@ -658,6 +620,38 @@ test_syncroot() {
 	assertTrue 'test failed: syncroot.txt not there as expected' "remote_file_exists 'syncroot.txt'"
 }
 
+test_submodule() {
+	submodule='sub'
+	file='file.txt'
+	mkdir "$submodule"
+	cd "$submodule"
+	touch "$file"
+	git init -q
+	git add .
+	git commit -m 'initial submodule commit' -q
+	cd ..
+	git submodule -q add "/$submodule" > /dev/null
+	git commit -m 'adding submodule' -q
+	init=$($GIT_FTP init)
+	assertTrue "test failed: $file not there as expected" "remote_file_exists '$submodule/$file'"
+}
+
+test_submodule_catchup() {
+	submodule='sub'
+	file='file.txt'
+	mkdir "$submodule"
+	cd "$submodule"
+	touch "$file"
+	git init -q
+	git add .
+	git commit -m 'initial submodule commit' -q
+	cd ..
+	git submodule -q add "/$submodule" > /dev/null
+	git commit -m 'adding submodule' -q
+	catchup=$($GIT_FTP catchup)
+	assertTrue "test failed: $submodule/.git-ftp.log not there as expected" "remote_file_exists '$submodule/.git-ftp.log'"
+}
+
 disabled_test_file_named_dash() {
 	cd $GIT_PROJECT_PATH
 	echo "foobar" > -
@@ -674,7 +668,10 @@ remote_file_exists() {
 }
 
 remote_file_equals() {
-	curl -s "$CURL_URL/$1" | diff - "$1" > /dev/null
+	local file="$1"
+	local remote="$2"
+	[ -z "$remote" ] && remote="$file"
+	curl -s "$CURL_URL/$remote" | diff - "$file" > /dev/null
 }
 
 assertContains() {
